package net.pingex.dcf.commands;

import net.pingex.dcf.commands.parser.BasicParser;
import net.pingex.dcf.commands.parser.ICommandParser;
import net.pingex.dcf.commands.parser.ParserException;
import net.pingex.dcf.modularity.PluginWrapper;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import sx.blah.discord.handle.impl.events.MessageReceivedEvent;
import java.lang.reflect.Method;
import java.util.List;

/**
 * Landing class for a MRE
 */
public class CommandHandler
{
	/**
	 * The one and only valid parser for now.
	 */
	public static final ICommandParser PARSER = new BasicParser();

	/**
	 * Logger
	 */
	private static final Logger LOGGER = LogManager.getLogger(CommandHandler.class);

	/**
	 * Landing method for a MRE
	 */
	public static void handle(MessageReceivedEvent event)
	{
		String command;
		List<String> arguments;

		try
		{
			command = PARSER.parseCommand(event.getMessage().getContent());
			arguments = PARSER.parseArguments(event.getMessage().getContent());
		}
		catch(ParserException e)
		{
			return; // Not a command
		}
		LOGGER.debug("Attempting to run command {} by user #{}.", command, event.getMessage().getAuthor().getID());
	}

	/**
	 * Registers a plugin's commands, if any
	 * @param pluginWrapper Target plugin
	 */
	public static void registerPlugin(PluginWrapper pluginWrapper)
	{
		LOGGER.debug("Registering commands for plugin {}.", pluginWrapper.getId());

		// Raw commands
		if(IWithCommands.class.isAssignableFrom(pluginWrapper.getInstance().getClass()))
		{
			LOGGER.debug("Plugin has commands (raw commands).");
			((IWithCommands) pluginWrapper.getInstance()).getCommands().forEach(CommandRegistry::registerCommand);
		}

		// Annotated commands
		if(IWithAnnotatedCommands.class.isAssignableFrom(pluginWrapper.getInstance().getClass()))
		{
			LOGGER.debug("Plugin has commands (annotated commands).");
			((IWithAnnotatedCommands) pluginWrapper.getInstance()).getAnnotatedObjects().forEach(e ->
			{
				for(Method i : e.getClass().getMethods())
					if(i.isAnnotationPresent(AnnotatedCommand.class))
						CommandRegistry.registerCommand(Command.buildFromAnnotatedCommand(i.getAnnotation(AnnotatedCommand.class), i, e));
			});
		}
	}

	/**
	 * Unregisters a plugin's commands, if any
	 * @param pluginWrapper Target plugin
	 */
	public static void unregisterPlugin(PluginWrapper pluginWrapper)
	{
		LOGGER.debug("Removing commands for plugin {}.", pluginWrapper.getId());

		// Raw commands
		if(IWithCommands.class.isAssignableFrom(pluginWrapper.getInstance().getClass()))
		{
			LOGGER.debug("Plugin has commands (raw commands).");
			((IWithCommands) pluginWrapper.getInstance()).getCommands().forEach(CommandRegistry::unregisterCommand);
		}

		// Annotated commands
		if(IWithAnnotatedCommands.class.isAssignableFrom(pluginWrapper.getInstance().getClass()))
		{
			LOGGER.debug("Plugin has commands (annotated commands).");
			((IWithAnnotatedCommands) pluginWrapper.getInstance()).getAnnotatedObjects().forEach(e ->
			{
				for(Method i : e.getClass().getMethods())
					if(i.isAnnotationPresent(AnnotatedCommand.class))
						CommandRegistry.unregisterCommandByName(i.getAnnotation(AnnotatedCommand.class).name());
			});
		}
	}
}
